@@ -1,4 +1,4 @@
-.noGenerics <- TRUE
+## we "need" S4 methods for dispatch on both (x,y)  .noGenerics <- TRUE
 .conflicts.OK <- TRUE
 
 .gmpVersion <- function() .Call(R_gmp_get_version)
